"""
Created on 12/dic/2013

@author: Marco Pompili
"""

import logging

from django import template
from django.conf import settings as user_settings

from sorl.thumbnail import get_thumbnail, delete

from django_easy_instagram import settings
from django_easy_instagram.scraper import instagram_profile_obj

logger = logging.getLogger(__name__)
handler = logging.StreamHandler()
formatter = logging.Formatter('%(name)s - %(levelname)s - %(message)s')

handler.setFormatter(formatter)
logger.addHandler(handler)

register = template.Library()


def get_profile_media(profile, page=0):
    """
    Parse a generated media object
    :param profile:
    :param page:
    :return:
    """
    try:
        edges = profile['entry_data']['ProfilePage'][page]['graphql']['user']['edge_owner_to_timeline_media']['edges']
        return [edge['node'] for edge in edges]
    except KeyError:
        logger.exception("path to profile media not found")


class InstagramUserRecentMediaNode(template.Node):
    """
    Template node for showing recent media from an user.
    """

    def __init__(self, var_name):
        self.var_name = var_name
        self.username = template.Variable(var_name)

    def render(self, context):
        try:
            profile = instagram_profile_obj(self.username.resolve(context))
        except template.base.VariableDoesNotExist:
            logger.warning(
                " variable name \"{}\" not found in context!"
                " Using a raw string as input is DEPRECATED."
                " Please use a template variable instead!".format(self.var_name)
            )

            profile = instagram_profile_obj(username=self.var_name)

        if profile:
            context['profile'] = profile
            context['recent_media'] = get_profile_media(profile)

        return ''


@register.tag
def instagram_user_recent_media(parser, token):
    """
    Tag for getting data about recent media of an user.
    :param parser:
    :param token:
    :return:
    """
    try:
        tagname, username = token.split_contents()

        return InstagramUserRecentMediaNode(username)
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires a single argument" % token.contents.split()[0]
        )


@register.filter(name='local_cache')
def local_cache(value, size="600x600"):
    im = get_thumbnail(value, size, crop='center', quality=settings.INSTAGRAM_QUALITY)
    return im.url



"""
Created on 04/sep/2016

@author: Marco Pompili
"""

import html5lib as html
import json
import logging
import requests

from socket import error as socket_error
from requests.exceptions import ConnectionError, HTTPError

from . import settings

SCRIPT_JSON_PREFIX = 18
SCRIPT_JSON_DATA_INDEX = 21


def instagram_scrape_profile(username):
    """
    Scrap an instagram profile page
    :param username:
    :return:
    """
    try:
        url = "https://www.instagram.com/{}/".format(username)
        headers = {
            "Accept":"text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "User-Agent": settings.INSTAGRAM_UA
        }
        if settings.INSTAGRAM_COOKIE:
            headers["Cookie"] = settings.INSTAGRAM_COOKIE
        page = requests.get(url, headers=headers)
        # Raise error for 404 cause by a bad profile name
        page.raise_for_status()
        return html.parse(page.content, treebuilder="dom")
    except HTTPError:
        logging.exception('user profile "{}" not found'.format(username))
    except (ConnectionError, socket_error) as e:
        logging.exception("instagram.com unreachable")


def instagram_profile_js(username):
    """
    Retrieve the script tags from the parsed page.
    :param username:
    :return:
    """
    try:
        tree = instagram_scrape_profile(username)
        return tree.getElementsByTagName("script")
    except AttributeError:
        logging.exception("scripts not found")
        return None


def instagram_profile_json(username):
    """
    Get the JSON data string from the scripts.
    :param username:
    :return:
    """
    scripts = instagram_profile_js(username)
    source = None

    if scripts:
        for script in scripts:
            if script.hasChildNodes():
                if script.firstChild.data[0:SCRIPT_JSON_PREFIX] == "window._sharedData":
                    source = script.firstChild.data[SCRIPT_JSON_DATA_INDEX:-1]

    return source


def instagram_profile_obj(username):
    """
    Retrieve the JSON from the page and parse it to a python dict.
    :param username:
    :return:
    """
    json_data = instagram_profile_json(username)

    return json.loads(json_data) if json_data else None



"""
Created on 24/gen/2014

@author: Marco Pompili
"""

import os
from setuptools import setup, find_packages

setup(
    name='django-easy-instagram',
    version='0.3.4',
    description='Instagram client for Django.',
    long_description_content_type='text/markdown',
    long_description=open('README.md', encoding='utf-8').read(),
    author='Tom Anthony, Marco Pompili',
    author_email='django@tomanthony.co.uk',
    license='BSD-3 License',
    url='https://github.com/TomAnthony/django-easy-instagram/',
    packages=find_packages(),
    platforms='any',
    include_package_data=True,
    install_requires=[
        'django>=1.6',
        'html5lib',
        'requests',
        'sorl-thumbnail',
        'Pillow',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Framework :: Django',
        'Framework :: Django :: 1.6',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.2',
    ],
)
